export run_control_2DOF

"""
	y,u,r = run_control(process, sysFB[, sysFF]; duration = 10, reference(t) = sign(sin(2π*t)))
Perform control experiemnt where the feedback and feedforward controllers are given by
`sysFB` and `sysFF`, both of type `StateSpace`.

`reference` is a reference generating function that accepts a scalar `t` (time in seconds) and outputs a scalar `r`, default is `reference(t) = sign(sin(2π*t))`.

The outputs `y,u,r` are the beam angle, control signal and reference respectively.

"""
function run_control_2DOF(P::AbstractProcess,sysFB, sysFF=nothing; duration = 10, reference = t->sign(sin(2π*t)))
	h 		= sampletime(P)
	y       = zeros(0:h:duration)
	u       = zeros(0:h:duration)
	r       = zeros(0:h:duration)

	sysFB   = minreal(sysFB)
	stateFB = init_sysfilter(sysFB)
	if sysFF != nothing
		sysFF   = minreal(sysFF)
		stateFF = init_sysfilter(sysFF)
	end

	function control(i)
		rf = sysFF == nothing ? r[i] : sysfilter!(stateFF, sysFF, r[i])
		e  = rf-y[i]
		ui = sysfilter!(stateFB, sysFB, e)
	end

	initialize(P)
	for (i,t) = enumerate(0:h:duration)
		@periodically h begin
			y[i]       = measure(P)
			r[i]       = reference(t)
			u[i]       = control(i) # y,r must be updated before u
			control(P, u[i])
		end
	end
	finalize(P)
	y,u,r
end
